#include <linux/linkage.h>
#include <asm/assembler.h>
#include <asm/arch/hardware.h>

#define DEBUG_UNDERFLOW 0



#define BUF_SIZE 0x00200000	
#define BUF_MASK 0x001FFFFF
	/*
	 * When the FIQ is initialized,
	 *
	 * r8_fiq  = ...            (read pointer, gets incremented)
	 * r9_fiq  = ...            (scratch...)
	 * r10_fiq = 0              (counts up to BUF_SIZE, this is mcp_readp)
	 * r11_fiq = ...            (scratch...)
	 * r12_fiq = 0xFF002000     (DAI registers within memory mapped by mmu)
	 * r13 = sp, stack pointer.  don't fuck with this.
	 * r14 = lr.  don't fuck with this either.
	 * r15 = pc.  definately don't fuck with the program counter.
	 *
	 * Note that the registers here are persistant from one 
	 * invocation of the FIQ to the next.  
	 *
	 */

	/*
	 *  This is what we need to do.
	 *
	 * If we have data to play, then write 4 samples to each left and then right channels
	 * UNTIL both FIFO's are half-full  (this part is important and probably where the current
	 * failure is).
	 *
	 * If there is no data, write zeros.  
	 * 
	 * Read 4 samples from receive buffer regardless.
	 *
	 * Clear overrun and underrun status bits in DAI status register.  
	 */

	.text
	.global	SYMBOL_NAME(dai_end)
ENTRY(dai_start)	

dai_start:	
	# is readp == writep?
	ldr	r11, .dai_writep
	ldr	r9, [r11]

	# yes?  
	cmp	r10, r9
	beq	play_silent

play_from_buffer:	
//
//  Read 4 samples from buffer and write them to the FIFO
//
	ldrh	r11, [r8, r10]
	str	r11, [r12, #0x80]
	add	r10, r10, #2
	
	ldrh	r11, [r8, r10]
	str	r11, [r12, #0x40]
	add	r10, r10, #2
	
	ldrh	r11, [r8, r10]
	str	r11, [r12, #0x80]
	add	r10, r10, #2
	
	ldrh	r11, [r8, r10]
	str	r11, [r12, #0x40]
	add	r10, r10, #2
	
	ldrh	r11, [r8, r10]
	str	r11, [r12, #0x80]
	add	r10, r10, #2
	
	ldrh	r11, [r8, r10]
	str	r11, [r12, #0x40]
	add	r10, r10, #2
	
	ldrh	r11, [r8, r10]
	str	r11, [r12, #0x80]
	add	r10, r10, #2
	
	ldrh	r11, [r8, r10]
	str	r11, [r12, #0x40]
	add	r10, r10, #2

	mov	r11, #BUF_SIZE
	cmp	r11, r10

	# wrap around
	movls	r10, #0x0

read_input:
	# Read from the incoming data registers.
	# just throw away the result
	ldr	r11, [r12, #0x80]
	ldr	r11, [r12, #0x40]
	ldr	r11, [r12, #0x80]
	ldr	r11, [r12, #0x40]
	ldr	r11, [r12, #0x80]
	ldr	r11, [r12, #0x40]
	ldr	r11, [r12, #0x80]
	ldr	r11, [r12, #0x40]


	# clear overrun and underrun bits in DAI status register
	mov	r11, #0xffffffff
	mov	r9, #0x100
	str	r11, [r12, r9]

	# Save dai_readp
	ldr	r11, .dai_readp
	str	r10, [r11]

	# return
	subs	pc, lr, #4


play_silent:
	# Write zeroes to the output registers.
	mov	r11, #0
	str	r11, [r12, #0x80]
	str	r11, [r12, #0x40]
	str	r11, [r12, #0x80]
	str	r11, [r12, #0x40]
	str	r11, [r12, #0x80]
	str	r11, [r12, #0x40]
	str	r11, [r12, #0x80]
	str	r11, [r12, #0x40]

#if DEBUG_UNDERFLOW
	ldr	r11, .dai_underflow
	mov	r9, #1
	str	r9, [r11]
#endif

	b	read_input


.dai_buf:	.long	SYMBOL_NAME(dai_buf)
.dai_readp:	.long	SYMBOL_NAME(dai_readp)
.dai_writep:	.long	SYMBOL_NAME(dai_writep)
#if DEBUG_UNDERFLOW
.dai_underflow:	.long	SYMBOL_NAME(dai_underflow)
#endif

.saved_regs:	.word	saved_area

SYMBOL_NAME(dai_end):

	
	.data
saved_area:
	.long	0
	.long	0
	.long	0
